<?php

namespace Drupal\gin_lb\TwigExtension;

use Drupal\Core\Template\Attribute;
use Drupal\Core\Template\TwigExtension;
use Twig\TwigFunction;

/**
 * A class providing Gin Twig function.
 *
 * @package Drupal\gin_lb\TwigExtension
 */
class GinLbExtension extends TwigExtension {

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('glb_classes', [$this, 'ginClasses']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'gin_lb';
  }

  /**
   * Replaces all theme classes with gin classes.
   *
   * @param \Drupal\Core\Template\Attribute $attribute
   *   Twig attributes.
   *
   * @return \Drupal\Core\Template\Attribute
   *   The replaced attributes.
   */
  public static function ginClasses(Attribute $attribute) {

    // Blacklist of all classes which should not be
    // prefixed. Javascript needs these classes.
    $black_list = [
      'layout-builder__add-section',
      'layout-builder__section',
      'layout__region-info',
      'layout-builder__region-label',
      'layout-builder__add-block',
      'form-actions',
      'views-element-container',
      'contextual-region',
      'focal-point',
      'focal-point-wrapper',
      'ui-patterns-variant-selector-section',
      'form-autocomplete',
    ];

    $black_list_prefixes = [
      'js-',
      'focal-point',
      'ui-patterns-variant-selector-section',
    ];
    if ($attribute->getClass() === NULL) {
      return $attribute;
    }
    $renamed_classes = [];

    foreach ($attribute->getClass() as $class_string) {
      // $attribute->getClass returns multiple classes as one class.
      // To prefix this classes - explode it.
      $classes = explode(' ', $class_string);
      foreach ($classes as $class) {
        $match_starts_with = FALSE;
        foreach ($black_list_prefixes as $black_list_prefix) {
          if (strpos($class, $black_list_prefix) === 0) {
            $match_starts_with = TRUE;
            break;
          }
        }
        if ($match_starts_with === FALSE && !in_array($class, $black_list)) {
          if (!empty($class)) {
            $renamed_classes[] = 'glb-' . $class;
          }
        }
        else {
          $renamed_classes[] = $class;
        }
      }
    }
    $attribute->setAttribute('class', $renamed_classes);
    return $attribute;
  }

}
